#!/usr/bin/ruby

require 'floppy_raw'
require 'gdt'

def scrub_regs16(boot, log)
  boot.puts "   xor ax, ax"
  boot.puts "   xor bx, bx" 
  boot.puts "   xor cx, cx"
  boot.puts "   xor dx, dx"
  log.puts "scrub_regs16(boot, log) completed"
end

def scrub_regs32(boot, log)
  boot.puts "   xor eax, eax"
  boot.puts "   xor ebx, ebx"
  boot.puts "   xor ecx, ecx"
  boot.puts "   xor edx, edx"
  log.puts "scrub_regs32(boot, log) completed"
end

def create_floppy_reset_code(boot, log)
  boot.puts "floppy_reset: "
  boot.puts "   sti"
  boot.puts "   int 0x13"
  boot.puts "   cmp ah, 0"
  boot.puts "   jne floppy_reset\n\n"
  log.puts "create_floppy_reset_code(boot, log) completed"
end

def create_floppy_stop_motor_code(boot, log)
  boot.puts "floppy_stop_motor: "
  boot.puts "   mov dx, 0x3F2"                  
  boot.puts "   mov al, 0x0C"           
  boot.puts "   out dx, al" 
  boot.puts "\n"
  log.puts "create_floppy_stop_motor_code(boot, log) completed"
end

def create_setup_gdt_code(boot, log)
  boot.puts "setup_gdt: "
  boot.puts "   cli"
  boot.puts "   lgdt [gdt_desc]"
  boot.puts "\n"
  log.puts "create_setup_gdt_code(boot, log) completed"
end

def create_setup_cr0_code(boot, log)
  boot.puts "setup_cr0: "
  boot.puts "   mov eax, cr0"
  boot.puts "   or al, 1"
  boot.puts "   mov cr0, eax"
  boot.puts "\n"
  log.puts "create_setup_cr0_code(boot, log) completed"
end

def create_far_jump_code(boot, log)
  boot.puts "far_jump: "
  boot.puts "   jmp 0x08:setup_stack\n\n"
  log.puts "create_far_jump_code(boot, log) completed"
end

def create_setup_stack_code(boot, log, stack)
  boot.puts "setup_stack: "
  scrub_regs32(boot, log)
  boot.puts "   mov ax, 0x10"
  boot.puts "   mov ds, ax"
  boot.puts "   mov es, ax"
  boot.puts "   mov fs, ax"
  boot.puts "   mov gs, ax"
  boot.puts "   mov ss, ax"
  boot.puts "   mov esp, " + stack
  boot.puts "\n"
  log.puts "create_setup_stack_code(boot, log, stack) completed"
end

def create_goto_kernel_code(boot, log, kernel_address)
  boot.puts "goto_kernel: "
  boot.puts "   jmp 0x" + kernel_address.to_s
  boot.puts "\n"
  log.puts "create_goto_kernel_code(boot, log, kernel_address) completed"
end

def create_bootstrap_signature_code(boot, log)
  boot.puts "times 510 - ($ - $$) db 0"
  boot.puts "dw 0xAA55"
  boot.puts "hlt"
  log.puts "create_bootstrap_signature_code(boot, log) completed"
end

f = File.new("boot.asm",  "w+")
f = File.new("boot.asm", File::CREAT|File::TRUNC|File::RDWR, 0644)
f.close
f = File.new("BootMe.log", "w+")
f = File.new("BootMe.log", File::CREAT|File::TRUNC|File::RDWR, 0644)
f.close
boot = File.new("boot.asm", "w")
log = File.new("BootMe.log", "w")

# add legal jibberish
boot.puts "; Author: 01000101 "
boot.puts "; Date: 7/27/2008 \n\n"
boot.puts "; This is an automatically configured OS bootloader created by the program BootMe. \n"
boot.puts "; The author relieves all rights to this software.  \n"
boot.puts "; This software, and it's sources, are public domain and express no liability of any kind. \n"
boot.puts "; This software is distributed 'as-is' and should not be used in any production environment. \n\n"

# set ORG and the label start 
boot.puts "[ORG 0x7C00]"
boot.puts "[bits 16]\n\n"
boot.puts "start: \n"
log.puts "Added ORG position and global start label used by common linkers"

# set all gen. purpose regs to zero
scrub_regs16(boot, log)
boot.puts "\n"

# write the floppy reset code
create_floppy_reset_code(boot, log)

system("clear")
# find out how many floppy sectors to load and where to put them (16-bit Seg:Off)
print "How many floppy cylinders should be loaded? \n"
print "[1-10]: "
nCylinders = gets.chomp

if nCylinders[0..1] == "0x"
  nCylinders = nCylinders.hex
else
  nCylinders = nCylinders.to_i
end

log.puts "Number of floppy cylinders = " + nCylinders.to_s + " (" + (nCylinders * 36).to_s + " sectors)"

if nCylinders == 0 || nCylinders >= 11
  log.puts "Number of floppy cylinders is not formatted correctly. (eg: 1 - 10 || 0x1 - 0xA)"
  exit
end

puts "\nWhere should these cylinders be loaded at? "
print "Segment [0-0xFFFF]: " 
segment = gets.chomp
log.puts "Segment = " + segment
if segment[0..1] != "0x" || segment.hex > 0xFFFF
  log.puts "Segment is not formatted correctly. (value not in hex notation or exceeds boundaries)"
  exit
end
print "Offset [0-0xFFFF]: " 
offset = gets.chomp
log.puts "Offset = " + offset
if offset[0..1] != "0x" || offset.hex > 0xFFFF
  log.puts "Offset is not formatted correctly. (value not in hex notation or exceeds boundaries)"
  exit
end

# write the code
create_floppy_load_sector_code(boot, log, nCylinders, segment, offset)
create_floppy_stop_motor_code(boot, log)
create_setup_gdt_code(boot, log)
create_setup_cr0_code(boot, log)

# calculate 32-bit address of thecreate_setup_gdt_code(boot) loaded sectors
temp = segment.hex
temp = temp << 4
temp = temp + offset.hex
pmode_address = temp.to_s(16)
temp = temp + 512
kernel_address = temp.to_s(16)

log.puts "pmode_address = 0x" + pmode_address
log.puts "kernel_address = 0x" + kernel_address

# jump to pmode
create_far_jump_code(boot, log)

boot.puts "[bits 32]"
# setup the stack
puts "\nWhere would you like the stack to be placed? "
printf "[0-0xFFFFFFFF]: "
stack = gets.chomp
log.puts "Stack = " + stack
if stack[0..1] != "0x" || stack.hex > 0xFFFFFFFF || stack.hex == 0
  log.puts "Stack is not formatted correctly. (Value not in hex notation or exceeds boundaries)"
end

create_setup_stack_code(boot, log, stack)
create_goto_kernel_code(boot, log, kernel_address)
# setup GDT descriptors
create_gdt_descriptor_code(boot, log)

# set bootstrap signature and fill up the rest of the sector with zeros
create_bootstrap_signature_code(boot, log)

boot.close
log.puts "successfully closed 'boot.asm'"
log.puts "closing 'BootMe.log'"
log.close

puts "\n"

